"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClusteredBrowser = void 0;
const puppeteer_cluster_1 = require("puppeteer-cluster");
const browser_1 = require("./browser");
var RenderType;
(function (RenderType) {
    RenderType["CSV"] = "csv";
    RenderType["PNG"] = "png";
})(RenderType || (RenderType = {}));
class ClusteredBrowser extends browser_1.Browser {
    constructor(config, log, metrics) {
        super(config, log, metrics);
        this.clusteringConfig = config.clustering;
        this.concurrency = puppeteer_cluster_1.Cluster.CONCURRENCY_BROWSER;
        if (this.clusteringConfig.mode === 'context') {
            this.concurrency = puppeteer_cluster_1.Cluster.CONCURRENCY_CONTEXT;
        }
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            const launcherOptions = this.getLauncherOptions({});
            this.cluster = yield puppeteer_cluster_1.Cluster.launch({
                concurrency: this.concurrency,
                maxConcurrency: this.clusteringConfig.maxConcurrency,
                timeout: this.clusteringConfig.timeout * 1000,
                puppeteerOptions: launcherOptions,
            });
            yield this.cluster.task(({ page, data }) => __awaiter(this, void 0, void 0, function* () {
                if (data.options.timezone) {
                    // set timezone
                    yield page.emulateTimezone(data.options.timezone);
                }
                try {
                    this.addPageListeners(page);
                    switch (data.renderType) {
                        case RenderType.CSV:
                            return yield this.exportCSV(page, data.options);
                        case RenderType.PNG:
                        default:
                            return yield this.takeScreenshot(page, data.options);
                    }
                }
                finally {
                    this.removePageListeners(page);
                }
            }));
        });
    }
    render(options) {
        return __awaiter(this, void 0, void 0, function* () {
            this.validateImageOptions(options);
            return this.cluster.execute({ options, renderType: RenderType.PNG });
        });
    }
    renderCSV(options) {
        return __awaiter(this, void 0, void 0, function* () {
            this.validateRenderOptions(options);
            return this.cluster.execute({ options, renderType: RenderType.CSV });
        });
    }
}
exports.ClusteredBrowser = ClusteredBrowser;
//# sourceMappingURL=clustered.js.map